#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Graphes en machine learning - énoncé
# 
# Ce notebook propose une série de graphes qu'on utilise fréquemment dans un notebook lorsqu'on fait du machine learning. Cela comprend notamment la courbe [ROC](https://fr.wikipedia.org/wiki/ROC) pour les problèmes de classification.

# In[1]:


# Cette instruction peut annuler l'effect de la suivante.
# On la place en premier.
get_ipython().run_line_magic('load_ext', 'pyensae')
get_ipython().run_line_magic('matplotlib', 'inline')


# In[2]:


import matplotlib.pyplot as plt
plt.style.use('ggplot')


# In[3]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# Le module utilise des données issues de [Wine Quality Data Set](https://archive.ics.uci.edu/ml/datasets/Wine+Quality) pour lequel on essaye de prédire la qualité du vin en fonction de ses caractéristiques chimiques.

# In[4]:


from pyensae.datasource import download_data, DownloadDataException
uci = "https://archive.ics.uci.edu/ml/machine-learning-databases/wine-quality/"
try:
    download_data("winequality-red.csv", url=uci)
    download_data("winequality-white.csv", url=uci)
except DownloadDataException:
    print("backup")
    download_data("winequality-red.csv", website="xd")
    download_data("winequality-white.csv", website="xd")


# In[5]:


get_ipython().run_line_magic('head', 'winequality-red.csv')


# In[6]:


import pandas
red_wine = pandas.read_csv("winequality-red.csv", sep=";")
red_wine["red"] = 1
white_wine = pandas.read_csv("winequality-white.csv", sep=";")
white_wine["red"] = 0
wines = pandas.concat([red_wine, white_wine])
wines.head()


# On découpe en base d'apprentissage, base de test :

# In[7]:


from sklearn.model_selection import train_test_split
X = wines[[c for c in wines.columns if c != "quality"]]
Y = wines["quality"]
x_train, x_test, y_train, y_test = train_test_split(X, Y, test_size=0.33, random_state=42)
type(x_train), type(y_train)


# In[8]:


wines.shape, x_train.shape, y_train.shape


# ## Exploration

# ### histogrammes
# 
# fonction [hist](http://pandas.pydata.org/pandas-docs/stable/generated/pandas.DataFrame.hist.html)

# In[9]:


get_ipython().run_line_magic('matplotlib', 'inline')


# In[10]:


import matplotlib.pyplot as plt


# In[11]:


plt.get_backend()


# In[12]:


wines.hist(figsize=(12,12))
print("-")


# Et plus particulièrement :

# In[13]:


wines.quality.hist(bins=7, figsize=(4,4))


# ### événements rares
# 
# Les queues de distributions sont des événements rares. De même que dans cet échantillon, la classe 9 est très peu représentée (voir ci-dessous). Lorsqu'on découpe le jeu de données en base d'apprentissage et base de test, ces événements ne sont pas uniformément distribués. Il est très peu probable qu'un classifieur quelconque arrive à prédire la classe 9. Il faudra traiter cette classe séparément. 
# 
# Il est difficile de dire si un prédicteur doit éviter d'utiliser des événements rares ou des valeurs aberrantes mais il est difficile de prévoir si la prédiction est généralisable. On choisit parfois de nettoyer les bases de ces événements, en les supprimant, en les lissant, en changeant d'échelle avec une échelle logarithmique.

# In[14]:


wines[["quality", "red"]].groupby("quality").count()


# ### corrélations
# 
# fonction [pairplot](https://stanford.edu/~mwaskom/software/seaborn/generated/seaborn.pairplot.html?highlight=pairplot#seaborn.pairplot)

# In[15]:


import seaborn
seaborn.pairplot(wines)


# fonction [clustermap](https://stanford.edu/~mwaskom/software/seaborn/generated/seaborn.clustermap.html), plus pratique quand il y a beaucoup de variables

# In[16]:


import seaborn
seaborn.set(font="monospace")
cmap = seaborn.diverging_palette(h_neg=210, h_pos=350, s=90, l=30, as_cmap=True)
seaborn.clustermap(wines.corr(), linewidths=.5, figsize=(13, 13), cmap=cmap)


# ## Classifieur
# 
# Pour le jeu [Wine Quality Data Set](https://archive.ics.uci.edu/ml/datasets/Wine+Quality), on essaye de prédire la note de l'expert qui est discrète et qui va de 1 à 9. On utilise l'exemple présent à la page [Decisions Trees](http://scikit-learn.org/stable/modules/tree.html) et un autre arbre de décision appris en faisant du boosting avec [AdaBoostClassifier](http://scikit-learn.org/stable/modules/generated/sklearn.ensemble.AdaBoostClassifier.html).

# In[17]:


from sklearn import tree
clf1 = tree.DecisionTreeClassifier(min_samples_leaf=10, min_samples_split=10)
clf1.fit(x_train, y_train)


# In[18]:


from sklearn.ensemble import AdaBoostClassifier
clf2 = AdaBoostClassifier(clf1,
                         algorithm='SAMME',
                         n_estimators=800,
                         learning_rate=0.5)
clf2.fit(x_train, y_train)


# On a besoin de spécifier le paramètre *min_samples_leaf* pour éviter de n'avoir qu'un élément dans chaque feuille de l'arbre. Ce n'est souvent pas très bon en terme de généralisation et l'arbre de décision retourne des probabilités de classification égale à 1 équivalent au ratio d'observations de la classe prédite dans cette feuille.

# In[19]:


clf1.classes_, "nb classes", len(clf1.classes_)


# In[20]:


clf2.classes_, "nb classes", len(clf2.classes_)


# Il n'y a aucune note en desous de 3 ou au-dessus de 9 dans la base d'apprentissage.

# ### matrice de confusion
# 
# La matrice de confusion est très utilisée dans les problèmes multi-classes comme celui-ci. Elle donne une bonne indication de la pertinence du modèle. L'inconvénient est qu'elle ne tient pas compte du score retourné par le modèle.
# 
# avec la fonction [confusion_matrix](http://scikit-learn.org/stable/modules/generated/sklearn.metrics.confusion_matrix.html)

# In[21]:


from sklearn.metrics import confusion_matrix
y_pred = clf1.predict(x_test)
conf1 = confusion_matrix(y_test, y_pred)
conf1


# In[22]:


y_pred = clf2.predict(x_test)
conf2 = confusion_matrix(y_test, y_pred)
conf2


# On peut considérer la précision par classe : coefficient de la diagonale divissée par l'ensemble des poids de la même ligne. Ou la précision globale : somme des coefficients de la diagonale sur l'ensemble des coefficients. L'erreur correspond à *1-precision*.

# In[23]:


[ conf1[i,i] / conf1[i,:].sum() for i in range(conf1.shape [0]) ]


# In[24]:


sum(conf1[i,i] for i in range(conf1.shape[0])) / conf1.sum()


# Et pour l'autre classifieur :

# In[25]:


[ conf2[i,i] / conf2[i,:].sum() for i in range(conf2.shape [0]) ]


# In[26]:


sum(conf2[i,i] for i in range(conf2.shape[0])) / conf2.sum()


# ### courbe ROC avec decision_function
# 
# voir [plot_roc](http://scikit-learn.org/stable/auto_examples/plot_roc.html)
# 
# La courbe ROC est l'outil de référence pour comparer les performances de deux classifiers en tenant compte du score de confiance qu'il retourne.
# 
# On superpose deux courbes ROC obtenue pour deux classes différentes : un vin de la classe *A* est bien classée si le modèle prédit *A*, il est mal classé s'il ne prédit pas *A*. Tout d'abord une courbe ROC s'appuie sur le score que retourne le classifieur avec la méthode *decision_function* excepté lorsque le modèle en question ne l'implémente pas. On ruse un peu pour construire un score :

# In[27]:


y_pred = clf2.predict(x_test)
y_prob = clf2.decision_function(x_test)
y_min = y_pred.min()
import numpy
y_score = numpy.array( [y_prob[i,p-y_min] for i,p in enumerate(y_pred)] ) 
y_score[:5], y_pred[:5], y_prob[:5,:]


# On crée les courbes ROC pour chacune des classes :

# In[28]:


from sklearn.metrics import roc_curve, auc

fpr = dict()
tpr = dict()
roc_auc = dict()
nb_obs = dict()

for i in clf2.classes_:
    fpr[i], tpr[i], _ = roc_curve(y_test == i, y_score)
    roc_auc[i] = auc(fpr[i], tpr[i])
    nb_obs[i] = (y_test == i).sum()
    
roc_auc, nb_obs


# On ajoute une dernière courbe ROC pour savoir si un élément est bien classé ou pas en balayant toutes les classes :

# In[29]:


i = "all"
fpr[i], tpr[i], _ = roc_curve(y_test == y_pred, y_score)
roc_auc[i] = auc(fpr[i], tpr[i])
nb_obs[i] = (y_test == y_pred).sum()

roc_auc_best, nb_obs_best = roc_auc, nb_obs
roc_auc, nb_obs


# In[30]:


import matplotlib.pyplot as plt
fig, axes = plt.subplots(1,2, figsize=(14,5), sharey=True)
for cl in (3,4,5):
    axes[0].plot(fpr[cl], tpr[cl], label='ROC classe %d (area = %0.2f)' % (cl, roc_auc[cl]))
for cl in ("all",):
    axes[1].plot(fpr[cl], tpr[cl], label='ROC classe (area = %0.2f)' % (roc_auc[cl]))
axes[0].legend()
axes[1].legend()


# In[31]:





# ### courbe ROC avec predict_proba
# 
# voir [plot_roc](http://scikit-learn.org/stable/auto_examples/plot_roc.html), on l'applique à l'arbre de décision

# In[32]:


y_pred = clf1.predict(x_test)
y_prob = clf1.predict_proba(x_test)
y_min = y_pred.min()
import numpy
y_score = numpy.array( [y_prob[i,p-y_min] for i,p in enumerate(y_pred)] ) 
y_score[:5]


# On crée les courbes ROC pour chacune des classes :

# In[33]:


from sklearn.metrics import roc_curve, auc

fpr = dict()
tpr = dict()
roc_auc = dict()
nb_obs = dict()

for i in clf1.classes_:
    fpr[i], tpr[i], _ = roc_curve(y_test == i, y_score)
    roc_auc[i] = auc(fpr[i], tpr[i])
    nb_obs[i] = (y_test == i).sum()
    
roc_auc, nb_obs


# On ajoute une dernière courbe ROC pour savoir si un élément est bien classé ou pas :

# In[34]:


i = "all"
fpr[i], tpr[i], _ = roc_curve(y_test == y_pred, y_score)
roc_auc[i] = auc(fpr[i], tpr[i])
nb_obs[i] = (y_test == y_pred).sum()

roc_auc, nb_obs


# In[35]:


import matplotlib.pyplot as plt
fig, axes = plt.subplots(1,2, figsize=(14,5), sharey=True)
for cl in (3,4,5):
    axes[0].plot(fpr[cl], tpr[cl], label='ROC classe %d (area = %0.2f)' % (cl, roc_auc[cl]))
for cl in ("all",):
    axes[1].plot(fpr[cl], tpr[cl], label='ROC classe (area = %0.2f)' % (roc_auc[cl]))
axes[0].legend()
axes[1].legend()


# Voir également [scikit-learn for TMVA Users](http://betatim.github.io/posts/sklearn-for-TMVA-users/).

# ## Alternatives à la courbe ROC

# ### un peu plus sur la courbe de ROC
# 
# La courbe ROC est parfois difficile à interpréter. Tout d'abord, elle suppose qu'un classifieur retourne une classe et un score. Le score détermine la confiance du modèle en sa prévision. Une classifieur dessine une frontière autour de classe, plus on est loin de cette frontière, plus on est confiant. Sa confiance est proche de 1. A l'inverse, si on cherche à prédire la classe d'une observation proche de la frontière, le classifier n'est pas très confiance. Sa confiance est proche de 0.
# 
# Le coin supérieur de droit de la courbe ROC a pour coordonnées (1,1) : dans ce cas, on valide toutes les réponses du classifieur. On valide toutes ses erreurs (abscisse = 100%) et toutes ses bonnes réponses (ordonnée = 100%). Si on décide que le classifieur ne peut pas répondre pour un score < 0.5, on va rejetter un certain nombre de réponse : on fera moins d'erreurs (x=100%-40%) et moins de bonnes réponses (y=100%-10%). La courbe passera par point de coordonnées (0.6,0.9). La courbe ROC est construite en faisant varier le seuil de validation des réponses du classifieur.

# In[36]:


from pyquickhelper.helpgen import NbImage
NbImage("roc_ex.png")


# Chaque point de la courbe est obtenu pour un seuil différent. Le seuil augmente (on valide moins de réponses) dans le sens de la flèche verte. Si le score d'un classifieur est pertinent, le nombre d'erreurs baisse plus vite que le nombre de bonnes réponses. Si le score n'est pas pertinent, la courbe ROC est proche de la diagonale. Lorsque le seuil de validation augmente, le ratio erreur sur bonnes réponses ne change pas.

# ### AUC
# 
# L'AUC ou [Area Under the Curve](https://en.wikipedia.org/wiki/Receiver_operating_characteristic#Area_under_the_curve) est l'aire sous la courbe ROC. Pour un classifieur binaire, si elle est égale à 0.5, cela signifie que le score classifieur ne donne aucune indication sur la qualité de la réponse (il est aléatoire). En-dessous, il y a probablement une erreur. L'AUC vérifie une autre propriété. Soit le score $p(x_1)$ le score d'une bonne réponse et $n(x_2)$ le score d'une mauvaise réponse, alors :
# 
# $\mathbb{P}(p(x_1) > n(x_2)) = AUC$
# 
# Autrement dit, la probabilité que le score d'une bonne réponse soit supérieure au score d'une mauvaise réponse est égale à l'AUC ([démonstration](http://www.xavierdupre.fr/site2013/documents/reports/roc.pdf)).

# ### distribution des scores
# 
# Pour illustrer cette AUC de façon différente, on peut tracer la distribution des scores pour les bonnes et les mauvaises réponses.

# In[37]:


from sklearn.ensemble import AdaBoostClassifier
clft = tree.DecisionTreeClassifier(min_samples_leaf=10, min_samples_split=10)
clfD = AdaBoostClassifier(clft,
                         algorithm='SAMME',
                         n_estimators=800,
                         learning_rate=0.5)
clfD.fit(x_train, y_train)


# In[38]:


from sklearn.metrics import auc, precision_recall_curve

y_predD = clfD.predict(x_test)
y_probD = clfD.predict_proba(x_test)
y_testn = y_test.values
y_minD = y_predD.min()
import numpy
y_scoreD = numpy.array( [y_probD[i,p-y_minD] for i,p in enumerate(y_predD)] )
y_scoreD[:5]


# In[39]:


positive_scores = y_scoreD[y_testn == y_predD]
negative_scores = y_scoreD[y_testn != y_predD]


# In[40]:


positive_scores.shape, negative_scores.shape


# In[41]:


from sklearn.metrics import roc_curve, auc

fpr = dict()
tpr = dict()
roc_auc = dict()
nb_obs = dict()

for i in clfD.classes_:
    fpr[i], tpr[i], _ = roc_curve(y_test == i, y_score)
    roc_auc[i] = auc(fpr[i], tpr[i])
    nb_obs[i] = (y_test == i).sum()
    
roc_auc, nb_obs


# In[42]:


import seaborn
ax = seaborn.distplot(positive_scores, rug=True, hist=True, label="+")
seaborn.distplot(negative_scores, rug=True, hist=True, ax=ax, label="-")
ax.legend()


# Le score de ce modèle n'est pas des plus discriminants puisqu'il existe une aire commune assez importante.

# ### taux de rappel et précision
# 
# Les taux de [rappel et de précision](https://fr.wikipedia.org/wiki/Pr%C3%A9cision_et_rappel) (precision recall) sont très utilisés en classification. Pour un seuil donné, la précision mesure le taux de réponse parmi les réponses validées (donc ayant un score au-dessus du seuil choisi), le taux de rappel mesure le taux de réponse validées.
# 
# La dénomination ``vrai positif``, ``faux positif``, ... est assez trompeuse. Comme le classifieur retourne un score de confiance, on décide de valider ou de rejeter sa réponse si le score est supérieur ou inférieur à ce seuil :
# 
# * si *score >= seuil*, on valide la réponse, qui est soit bonne (*TP : True Positive*), soit fausse (*FP : False Positive*)
# * si *score < seuil*, on rejete la réponse qui est soit bonne (*FN : False Negative*) soit fausse (*TN : True Negative*)
# 
# La présicion est définie comme étant le nombre de réponses justes sur le nombre de réponses validées :
# 
# $precision = \frac{TP}{TP + FP}$ et $rappel = \frac{TP}{TP + FN}$
# 
# On utilise la fonction [precision_recall_curve](http://scikit-learn.org/stable/modules/generated/sklearn.metrics.precision_recall_curve.html).

# In[43]:


from sklearn.ensemble import AdaBoostClassifier
clft = tree.DecisionTreeClassifier(min_samples_leaf=10, min_samples_split=10)
clf4 = AdaBoostClassifier(clft,
                         algorithm='SAMME',
                         n_estimators=800,
                         learning_rate=0.5)
clf4.fit(x_train, y_train)


# In[44]:


from sklearn.metrics import auc, precision_recall_curve

y_pred4 = clf4.predict(x_test)
y_prob4 = clf4.predict_proba(x_test)
y_min4 = y_pred4.min()
import numpy
y_score4 = numpy.array( [y_prob4[i,p-y_min4] for i,p in enumerate(y_pred4)] )
y_score4[:5]

precision = dict()
recall = dict()
threshold = dict()
nb_obs = dict()

for i in clf4.classes_:
    precision[i], recall[i], threshold[i] = precision_recall_curve(y_test == i, y_score4)
    nb_obs[i] = (y_test == i).sum()

i = "all"
precision[i], recall[i], threshold[i] = precision_recall_curve(y_test == y_pred4, y_score4)
nb_obs[i] = (y_test == y_pred4).sum()


# In[45]:


import matplotlib.pyplot as plt
fig, axes = plt.subplots(1,2, figsize=(14,5))
for cl in (3,4,5,6,7,'all'):
    axes[0].plot(precision[cl], recall[cl], label='Precision/Rappel classe %s' % str(cl))
cl = 'all'
axes[1].plot(threshold[cl], recall[cl][1:], label='recall - all')
axes[1].plot(threshold[cl], precision[cl][1:], label='precision - all')
axes[1].set_xlabel("threshold")
axes[0].set_xlabel("precision")
axes[0].set_ylabel("recall")
axes[0].legend()
axes[1].legend()


# Il paraît difficile de faire montrer la précision dans baisser beaucoup le rappel. Cette courbe est inconsistance lorsque le rappel est très bas car la précision est estimée sur peu d'observations.

# ## Régression

# In[46]:


from sklearn import linear_model
clr1 = linear_model.LinearRegression()
clr1.fit(x_train, y_train)


# In[47]:


from sklearn import tree
clr2 = tree.DecisionTreeRegressor(min_samples_leaf=10)
clr2.fit(x_train, y_train)


# ### cross validation

# In[48]:


from sklearn.model_selection import cross_val_score
cross_val_score(clr1, x_train, y_train, cv=10)


# In[49]:


cross_val_score(clr2, x_train, y_train, cv=10)


# ### réprésentation des erreurs
# 
# Difficile de représentation des erreurs lorsque le problème est à plusieurs dimensions. On trie les erreurs de façon croissante.

# In[50]:


y_pred = clr1.predict(x_test)
diff1 = (y_test * 1.0 - y_pred).sort_values()
diff1.head()


# In[51]:


y_pred = clr2.predict(x_test)
diff2 = (y_test * 1.0 - y_pred).sort_values()
diff2.head()


# In[52]:


import matplotlib.pyplot as plt
fig, axes = plt.subplots(1,1, figsize=(16,4))
axes.plot(list(range(len(diff1))), list(diff1),"r-", markersize = 3, label="linear")
axes.plot(list(range(len(diff2))), list(diff2),"g-", markersize = 3, label="tree")
axes.plot(list(range(len(diff1))), [0 for i in range(len(diff1))], "b--")
axes.legend()


# L'erreur est reliée à l'aire sous la courbe.

# ### régression 1d et tendance

# In[53]:


x_train1d = x_train.copy()
x_train1d["const"] = 1.0
train_1d = x_train1d[["alcohol","const", "residual sugar"]]
train_1d.head()


# In[54]:


from sklearn import linear_model
clr = linear_model.LinearRegression()
clr.fit(train_1d[["const", "alcohol"]], train_1d["residual sugar"])


# In[55]:


pred = clr.predict(train_1d[["const", "alcohol"]])
graph = train_1d.copy()
graph["trend"] = pred
graph.head()


# In[56]:


ax = graph.plot(x="alcohol", y="residual sugar", kind="scatter", label="residual sugar")
graph.sort_values("alcohol").plot(x="alcohol", y="trend", ax=ax, color="green", label="trend")


# On peut aussi jouer avec la fonction [add_trend](http://statsmodels.sourceforge.net/devel/generated/statsmodels.tsa.tsatools.add_trend.html) de [statsmodels](http://statsmodels.sourceforge.net/devel/index.html).

# In[57]:





# ### Exercice 1 : créer une fonction pour automatiser la création de ce graphe

# In[58]:





# ## Autres variantes

# ### On agrège les classes 8 et 9, 3 et 4
# 
# On agrége ces classes pour lesquels on manque d'observations.

# In[59]:


y_train89 = y_train.copy()
y_test89 = y_test.copy()
y_train89[y_train89==9] = 8
y_test89[y_test89==9] = 8
y_train89[y_train89==3] = 4
y_test89[y_test89==3] = 4


# In[60]:


from sklearn.ensemble import AdaBoostClassifier
clft = tree.DecisionTreeClassifier(min_samples_leaf=10, min_samples_split=10)
clf89 = AdaBoostClassifier(clft,
                         algorithm='SAMME',
                         n_estimators=800,
                         learning_rate=0.5)
clf89.fit(x_train, y_train89)


# In[61]:


from sklearn.metrics import confusion_matrix
y_pred = clf89.predict(x_test)
conf89 = confusion_matrix(y_test89, y_pred)
conf89


# In[62]:


from sklearn.metrics import roc_curve, auc

y_pred89 = clf89.predict(x_test)
y_prob89 = clf89.predict_proba(x_test)
y_min89 = y_pred89.min()
import numpy
y_score89 = numpy.array( [y_prob89[i,p-y_min89] for i,p in enumerate(y_pred89)] ) 
y_score89[:5]

fpr89 = dict()
tpr89 = dict()
roc_auc89 = dict()
nb_obs89 = dict()

for i in clf89.classes_:
    fpr89[i], tpr89[i], _ = roc_curve(y_test89 == i, y_score89)
    roc_auc89[i] = auc(fpr89[i], tpr89[i])
    nb_obs89[i] = (y_test89 == i).sum()
    
i = "all"
fpr89[i], tpr89[i], _ = roc_curve(y_test89 == y_pred89, y_score89)
roc_auc89[i] = auc(fpr89[i], tpr89[i])
nb_obs89[i] = (y_test89 == y_pred89).sum()    
    
roc_auc89, nb_obs89


# A comparer avec le meilleur classifieur :

# In[63]:


roc_auc_best, nb_obs_best


# In[64]:


import matplotlib.pyplot as plt
fig, axes = plt.subplots(1,2, figsize=(14,5), sharey=True)
for cl in (4,5,6,7,8):
    axes[0].plot(fpr89[cl], tpr89[cl], label='ROC classe %d (area = %0.2f)' % (cl, roc_auc89[cl]))
for cl in ("all",):
    axes[1].plot(fpr89[cl], tpr89[cl], label='ROC classe (area = %0.2f)' % (roc_auc89[cl]))
axes[0].legend()
axes[1].legend()


# ### On normalise les features
# 
# Même si cette méthode n'apporte pas grand chose dans ce cas, la normalisation par logarithme aide le classifieur lorsque les distributions ont des queues épaisses ou des points aberrants : cela donne moins d'importance aux événements extrêmes.

# In[65]:


import math
x_train_n = x_train.copy()
x_test_n = x_test.copy()
x_train_n = x_train_n.applymap(lambda x: math.log(1+x))
x_test_n = x_test_n.applymap(lambda x: math.log(1+x))


# In[66]:


x_train_n.hist(figsize=(14,14))


# In[67]:


from sklearn.ensemble import AdaBoostClassifier
clft = tree.DecisionTreeClassifier(min_samples_leaf=10, min_samples_split=10)
clfn = AdaBoostClassifier(clft,
                         algorithm='SAMME',
                         n_estimators=800,
                         learning_rate=0.5)
clfn.fit(x_train_n, y_train)


# In[68]:


from sklearn.metrics import confusion_matrix
y_predn = clfn.predict(x_test_n)
confn = confusion_matrix(y_test, y_predn)
confn


# In[69]:


from sklearn.metrics import roc_curve, auc

y_predn = clfn.predict(x_test_n)
y_probn = clfn.predict_proba(x_test_n)
y_minn = y_predn.min()
import numpy
y_scoren = numpy.array( [y_probn[i,p-y_minn] for i,p in enumerate(y_predn)] ) 
y_scoren[:5]

fpr = dict()
tpr = dict()
roc_auc = dict()
nb_obs = dict()

for i in clfn.classes_:
    fpr[i], tpr[i], _ = roc_curve(y_test == i, y_scoren)
    roc_auc[i] = auc(fpr[i], tpr[i])
    nb_obs[i] = (y_test == i).sum()
    
i = "all"
fpr[i], tpr[i], _ = roc_curve(y_test == y_predn, y_scoren)
roc_auc[i] = auc(fpr[i], tpr[i])
nb_obs[i] = (y_test == y_predn).sum()    
    
roc_auc, nb_obs


# In[70]:


import matplotlib.pyplot as plt
fig, axes = plt.subplots(1,2, figsize=(14,5), sharey=True)
for cl in (4,5,6,7,8):
    axes[0].plot(fpr[cl], tpr[cl], label='ROC classe %d (area = %0.2f)' % (cl, roc_auc[cl]))
for cl in ("all",):
    axes[1].plot(fpr[cl], tpr[cl], label='ROC classe (area = %0.2f)' % (roc_auc[cl]))
axes[0].legend()
axes[1].legend()
axes[0].set_ylabel("True Positive Rate")
axes[0].set_xlabel("False Positive Rate")
axes[1].set_xlabel("False Positive Rate")


# ### Classifieur pour les classes 8-9

# In[71]:


from sklearn.naive_bayes  import GaussianNB
clfs89 = GaussianNB()
clfs89.fit(x_train, y_train89)


# In[72]:


from sklearn.metrics import confusion_matrix
y_pred89 = clfs89.predict(x_test)
confs89 = confusion_matrix(y_test89, y_pred89)
confs89


# In[73]:


from sklearn.metrics import roc_curve, auc

y_preds89 = clfs89.predict(x_test)
y_probs89 = clfs89.predict_proba(x_test)
y_mins89 = y_preds89.min()
import numpy
y_scores89 = numpy.array( [y_probs89[i,p-y_mins89] for i,p in enumerate(y_preds89)] ) 
y_scores89[:5]

fpr = dict()
tpr = dict()
roc_auc = dict()
nb_obs = dict()

for i in clfs89.classes_:
    fpr[i], tpr[i], _ = roc_curve(y_test89 == i, y_scores89)
    roc_auc[i] = auc(fpr[i], tpr[i])
    nb_obs[i] = (y_test89 == i).sum()
    
i = "all"
fpr[i], tpr[i], _ = roc_curve(y_test89 == y_preds89, y_scores89)
roc_auc[i] = auc(fpr[i], tpr[i])
nb_obs[i] = (y_test89 == y_preds89).sum()    
    
roc_auc, nb_obs


# In[74]:


get_ipython().run_line_magic('matplotlib', 'inline')


# In[75]:


import matplotlib.pyplot as plt
fig, axes = plt.subplots(1,2, figsize=(14,5), sharey=True)
for cl in clfs89.classes_:
    axes[0].plot(fpr[cl], tpr[cl], label='ROC classe %d (area = %0.2f)' % (cl, roc_auc[cl]))
for cl in ("all",):
    axes[1].plot(fpr[cl], tpr[cl], label='ROC classe (area = %0.2f)' % (roc_auc[cl]))
axes[0].legend()
axes[1].legend()
axes[0].set_ylabel("True Positive Rate")
axes[0].set_xlabel("False Positive Rate")
axes[1].set_xlabel("False Positive Rate")


# ## Autres modules

# ### XGBoost
# 
# Le module [XGBoost](http://xgboost.readthedocs.org/en/latest/python) est utilisé dans la plupart des compétitions [Kaggle](https://www.kaggle.com/).

# In[76]:


import xgboost


# In[77]:


try:
    clxg = xgboost.XGBClassifier(max_depth=10, min_child_weight=5, nthread=3, n_estimators=800,
                                 colsample_bytree=0.5)
except Exception:
    # Newer version of XGBoost have different parameters.
    clxg = xgboost.XGBClassifier(max_depth=10, min_child_weight=5, nthread=3, n_estimators=800,
                                 colsample_bytree=2)
clxg.fit(x_train, y_train)


# In[78]:


from sklearn.metrics import confusion_matrix
y_pred = clxg.predict(x_test)
confxg = confusion_matrix(y_test, y_pred)
confxg


# In[79]:


from sklearn.metrics import roc_curve, auc

y_predxg = clxg.predict(x_test)
y_probxg = clxg.predict_proba(x_test)
y_minxg = y_predxg.min()
import numpy
y_scorexg = numpy.array( [y_probxg[i,p-y_minxg] for i,p in enumerate(y_predxg)] ) 
y_scorexg[:5]

fpr = dict()
tpr = dict()
roc_auc = dict()
nb_obs = dict()

for i in clxg.classes_:
    fpr[i], tpr[i], _ = roc_curve(y_test == i, y_scorexg)
    roc_auc[i] = auc(fpr[i], tpr[i])
    nb_obs[i] = (y_test == i).sum()
    
i = "all"
fpr[i], tpr[i], _ = roc_curve(y_test == y_predxg, y_scorexg)
roc_auc[i] = auc(fpr[i], tpr[i])
nb_obs[i] = (y_test == y_predxg).sum()    
    
roc_auc, nb_obs


# In[80]:


import matplotlib.pyplot as plt
fig, axes = plt.subplots(1,2, figsize=(14,5), sharey=True)
for cl in (3,4,5,6,7,8):
    axes[0].plot(fpr[cl], tpr[cl], label='ROC classe %d (area = %0.2f)' % (cl, roc_auc[cl]))
for cl in ("all",):
    axes[1].plot(fpr[cl], tpr[cl], label='ROC classe (area = %0.2f)' % (roc_auc[cl]))
axes[0].legend()
axes[1].legend()
axes[0].set_ylabel("True Positive Rate - XGBOOST")
axes[0].set_xlabel("False Positive Rate")
axes[1].set_xlabel("False Positive Rate")


# ### LightGBM
# 
# Le module [LightGBM](https://github.com/Microsoft/LightGBM) est un équivalent open source écrit par Microsoft. Il est plus rapide que XGBoost dans la plupart des cas.

# In[81]:


import lightgbm


# In[82]:


clgbm = lightgbm.LGBMClassifier(max_depth=10, min_child_weight=5, nthread=3, n_estimators=800,
                             colsample_bytree=0.5)
clgbm.fit(x_train, y_train)


# In[83]:


from sklearn.metrics import confusion_matrix
y_pred = clgbm.predict(x_test)
confgbm = confusion_matrix(y_test, y_pred)
confgbm


# In[84]:


from sklearn.metrics import roc_curve, auc

y_predgbm = clgbm.predict(x_test)
y_probgbm = clgbm.predict_proba(x_test)
y_mingbm = y_predgbm.min()
import numpy
y_scoregbm = numpy.array( [y_probgbm[i,p-y_minxg] for i,p in enumerate(y_predgbm)] ) 
y_scoregbm[:5]

fpr = dict()
tpr = dict()
roc_auc = dict()
nb_obs = dict()

for i in clxg.classes_:
    fpr[i], tpr[i], _ = roc_curve(y_test == i, y_scorexg)
    roc_auc[i] = auc(fpr[i], tpr[i])
    nb_obs[i] = (y_test == i).sum()
    
i = "all"
fpr[i], tpr[i], _ = roc_curve(y_test == y_predxg, y_scorexg)
roc_auc[i] = auc(fpr[i], tpr[i])
nb_obs[i] = (y_test == y_predxg).sum()    
    
roc_auc, nb_obs


# In[85]:


import matplotlib.pyplot as plt
fig, axes = plt.subplots(1,2, figsize=(14,5), sharey=True)
for cl in (3,4,5,6,7,8):
    axes[0].plot(fpr[cl], tpr[cl], label='ROC classe %d (area = %0.2f)' % (cl, roc_auc[cl]))
for cl in ("all",):
    axes[1].plot(fpr[cl], tpr[cl], label='ROC classe (area = %0.2f)' % (roc_auc[cl]))
axes[0].legend()
axes[1].legend()
axes[0].set_ylabel("True Positive Rate - LightGBM")
axes[0].set_xlabel("False Positive Rate")
axes[1].set_xlabel("False Positive Rate")


# ## Exercice 2 : simplifier l'apprentissage  de chaque modèle

# In[86]:





# ## Exercice 3 : grid_search
# 
# Considérer un modèle et estimer au mieux ses paramètres.

# In[87]:





# In[88]: